/*!
CTexture.h
 Menu Disc Demo
 
 Author: Jeroen Verbeek
 Copyright (c) Jeroen Verbeek, 2003-2004. All rights reserved.
 
 This software is provided without any guarantee.
 No responsibility taken for any problems caused by the use if this software.
 
 Anyone is authorised to use this program code and methods without warranty,
 all that is required is that you contact me (jeroen@happymakinggames.com),
 and tell me that you use it and what you use it in -
 all I want is to know is who uses it and where.
 
 I will not ask for payment in any way: this code is free.
 If you encounter a bug with the code, please tell me about it.
 If you have a bug-fix for the code please provide me with it.
 If you think of an improvement please tell me about it.
 You are not required to provide me with bug reports, bug fixes or improvements.
 If you require me to perform any work related to this code or
 not related to this code, please contact me: I am always open to offers.
 
 Pie menus are in no way my original idea, I stumbled on the idea a few years ago,
 specifically Don Hopkins' http://catalog.com/hopkins/piemenus/piemenus-hopkins.html.
 I am a big fan of circular menus (disc menus).
 I am of the opinion that they provide the quickest method of selecting an action
 from a structured tree of actions. A menu of actions organised following this method
 will, in time, allow almost "blind selection" of actions. Each action selection
 requires the same distance of mouse movement, just in a slightly diferent angle.
 In my opinion this seems to work best with menu discs of no more than 8 items.
 There is plenty of research to be found through the internet regarding the 
 advantages of circular menus, for example: muscle memory.
 Some websites I recommend you look at are:
 
 http://catalog.com/hopkins/piemenus/piemenus-hopkins.html
 
 http://www.piemenus.com/
 
 http://www.sm.luth.se/~david/carl/www/argonaut.html
 
 Contact: Jeroen Verbeek, jeroen@happymakinggames.com, Sydney NSW Australia
 */
/*-----------------------------------------------------------------------------------------------*/
#ifndef _CTEXTURE_H_
#define _CTEXTURE_H_



/*-----------------------------------------------------------------------------------------------*/
/*!
\brief	Simple texture class.
 
Texture class for basic texture handling.
This is a base class for all file format specific texture loaders, for example: CTextureTGA.
 */
/*-----------------------------------------------------------------------------------------------*/
class CTexture
{
public:
	CTexture(void){m_ImageDataKept = false;}
	virtual ~CTexture(){;}
	
	//! Each derived class handles it's own graphic file format.
	virtual int Load(char *name, int id, int textureMode, bool keepimagedata) = 0;

	unsigned char *GetImageData(void) const {return m_ImageData;}
	int GetImageWidth(void) const {return m_ImageWidth;}
	int GetImageHeight(void) const {return m_ImageHeight;}
	int GetImageDepth(void) const {return m_ImageDepth;}
	int GetImageSize(void) const {return m_Size;}
	int IsPowerOf2(int n) const {return (n & -n) == n;}

protected:
	int m_ImageWidth, m_ImageHeight, m_ImageDepth, m_Size, m_ImagePixelSize;
    bool m_ImageDataKept;
	unsigned char *m_ImageData;
	GLenum m_TextureFormat;
};


/*-----------------------------------------------------------------------------------------------*/
/*! 
Simple header for TGA files.
*/
/*-----------------------------------------------------------------------------------------------*/
typedef struct TGA_HEADER
{
	char extra;
	unsigned char colourmap;
	unsigned char imagetype;
	char filler[9];
	unsigned char width[2];
	unsigned char height[2];
	unsigned char depth;
};


/*-----------------------------------------------------------------------------------------------*/
/*! 
Texture sub-class for TGA files.
Does not handle RLE compressed files (yet).
*/
/*-----------------------------------------------------------------------------------------------*/
class CTextureTGA : CTexture
{
public:
	CTextureTGA();
	~CTextureTGA();

	//! Local implementation of CTexture:Load function to load TGA files only.
	int Load(char *name, int id,int textureMode, bool keepimagedata);

private:
	unsigned char *ReadData(FILE *file);
};




#endif